/*
 * torture_smb.c
 * Copyright (C) Richard Sharpe, 1996
 *
 * We torture test SMB servers ...
 */

#include <stdlib.h>
#include <string.h>
#include <rfcnb.h>
#include <smblib.h>
#include <tcp.h>

#define DEFAULT "msfax"

char data[65536] = "Now is the time for all birds to roost.";
char *SMB_Prots[] = {
     "PC NETWORK PROGRAM 1.0",
     "MICROSOFT NETWORKS 1.03",
     "MICROSOFT NETWORKS 3.0",
     "DOS LANMAN1.0",
     "LANMAN1.0",
     "DOS LM1.2X002",
     "LM1.2X002",
     "DOS LANMAN2.1",
     "LANMAN2.1",
     "Samba",
     "NT LM 0.12",
     "NT LANMAN 1.0",
     NULL
   };

char *prots_to_use[1000];       /* We can pass these in ... */
int   prot_count = 0;

int verbose = FALSE;

char *SMB_Prots2[] = {
     "PC NETWORK PROGRAM 1.0",
     "MICROSOFT NETWORKS 1.03",
     "MICROSOFT NETWORKS 3.0",
     "DOS LANMAN1.0",
     "LANMAN1.0",
     "DOS LM1.2X002",
     "LM1.2X002",
     "DOS LANMAN2.1",
     "LANMAN2.1",
     "NT LM 0.12",
     "NT LANMAN 1.0",
     NULL
   };

void do_error (int error)
{
  char err_str[1024];

  if (error == SMBlibE_Remote)
  {
    error = SMB_Get_Last_SMB_Err();
    SMB_Get_SMB_Error_Msg (SMBlib_Error_Class (error),
                           SMBlib_Error_Code (error),
                           err_str, sizeof(err_str)-1);
  }
  else
    SMB_Get_Error_Msg (error, err_str, sizeof(err_str)-1);

  printf ("  %s\n", err_str);
}

/* Some torture tests ...
 */

/*
 * Torture_mult_neg connects to the server and then does multiple
 * negotiates
 */
int torture_mult_neg (char *server)
{
  struct SMB_Connect *con;
  int    prot;
  char **use_prots;

  fprintf (stderr, "Test: Multiple Negotiates\n");
  fprintf (stderr, "Testing whether or not server %s can handle multiple negotiates\n", server);

  con = SMB_Connect_Server (NULL, server);

  if (!con)
  {       
    fprintf (stderr, "Unable to connect to server \"%s\" ... \n", server);
    do_error (SMB_Get_Last_Error());

    fprintf (stderr, "Since we could not connect to the server, no "
             "more tests will be run in this group.\n");
    return (-1);
  }

  /* Now we negotiate and negotiate again ... */

  if (prot_count > 0)
       use_prots = prots_to_use;
  else use_prots = SMB_Prots;

  fprintf (stderr, "Now we negotiate a protocol ... \n");

  if (SMB_Negotiate (con, use_prots) < 0)
  {
    fprintf (stderr, "Unable to negotiate a protocol ...\n");
    do_error (SMB_Get_Last_Error());

    /* Well, it did not like that list of protocols, let's see if it likes
     * a smaller list
     */
    if (SMB_Negotiate (con, SMB_Prots2) < 0)
    {
      fprintf (stderr, "Unable to negotiate a protocol a from a smaller list...\n");
      do_error (SMB_Get_Last_Error());
      fprintf (stderr, "Since they don't like our protocols, let's go home.\n");
      return (-1);
    }
  }

  prot = SMB_Get_Protocol_IDX (con);

  fprintf (stderr, "  The protocol accepted was %i, %s\n", prot, use_prots[prot]);
  fprintf (stderr, "  The Max Buffer Size is: %i\n", SMB_Get_Max_Buf_Siz (con));

  fprintf (stderr, "Now we negotiate a protocol again ... \n");

  if (SMB_Negotiate (con, use_prots) < 0)
  {
    fprintf (stderr, "Unable to negotiate a protocol a second time...\n");
    do_error (SMB_Get_Last_Error());
    /* Ignore any error ... */
  }
  else
  {
    prot = SMB_Get_Protocol_IDX (con);

    fprintf (stderr, "  The protocol accepted was %i, %s\n", prot, use_prots[prot]);
  }
  SMB_Discon (con, FALSE);
  fprintf (stderr, "Test: Multiple Negotiates complete.\n");
  return (0);
}

/*
 * Send a long negotiate SMB and see what happens ...
 */
int torture_long_neg (char *server)
{
  struct SMB_Connect *con;
  int    prot, i;
  char **prot_list;

  fprintf (stderr, "Test: Long Negotiate SMBs\n");
  fprintf (stderr, "Testing whether or not server %s can handle multiple negotiates\n", server);
  fprintf (stderr, "If we hang here for more than 10 seconds, or we timeout,");
  fprintf (stderr, " the server does\n not like long SMBs ");
  fprintf (stderr, "and stalled the connection.\n");

  con = SMB_Connect_Server (NULL, server);
  if (!con)
  { 
    fprintf (stderr, "Unable to connect to server \"%s\" ... \n", server);
    do_error (SMB_Get_Last_Error());
    fprintf (stderr, "Since we could not connect to the server, no more "
             "tests will be run in this group.\n");
    return (-1);
  }

  /* Now we negotiate a protocol ... */

  fprintf (stderr, "Now we negotiate a protocol with a long SMB ... \n");

  if ((prot_list = calloc (1001, sizeof(char*))) == NULL)
  {
    fprintf (stderr, "Could not allocate space for the protocols ...\n");
    exit (1);
  }

  for (i = 0; i < 992; i++)
      prot_list[i] = "One really stupid protocol name";

  for (i = 992; i < 1000; i++)
      prot_list[i] = SMB_Prots[i-992];

  if (SMB_Negotiate (con, prot_list) < 0)
  {
    fprintf (stderr, "Unable to negotiate a protocol ...\n");
    do_error (SMB_Get_Last_Error());
    return (-1);
  }

  prot = SMB_Get_Protocol_IDX (con);

  fprintf (stderr, "  The protocol accepted was %i, %s\n",
           prot, prot_list[prot]);

  SMB_Discon (con, FALSE);
  fprintf (stderr, "Test: Long Negotiate SMBs complete.\n");
  return (0);
}

int torture_template (char *server)
{
  struct SMB_Connect *con;

  fprintf (stderr, "Test: Multiple Negotiates\n");
  fprintf (stderr, "Testing whether or not server %s can handle multiple negotiates\n", server);

  con = SMB_Connect_Server (NULL, server);
  if (!con)
  { 
    fprintf (stderr, "Unable to connect to server \"%s\" ... \n", server);
    do_error (SMB_Get_Last_Error());
    fprintf (stderr, "Since we could not connect to the server, no more "
             "tests will be run in this group.\n");
    return (-1);
  }

  SMB_Discon (con, FALSE);
  fprintf (stderr, "Test: Multiple Negotiates complete.\n");
  return (0);
}

void usage (char *prog)
{
  fprintf (stderr, "%s [-v -n <protocol>] server\n", prog);
  fprintf (stderr, "Where: -v means verbose listing of info\n");
  fprintf (stderr, "       -n <protocol> means add protocol to the list.");
  fprintf (stderr, "                     If you use -n, then you have to\n");
  fprintf (stderr, "                     specify all the protocols to use.\n");
}

int main (int argc, char **argv)
{
  int    opt;
  char   server[80];
  char   password[80];

  while ((opt = getopt (argc, argv, "n:b:e:s:p:u:lv")) != EOF)
  {
    switch (opt)
    {
      case 'n':             /* Pick up another protocol string to negotiate */
           prots_to_use[prot_count] = optarg;
           prot_count = prot_count + 1;
           break;

      case 'v':             /* Set verbose flag */
           verbose = TRUE;
           dbug_init();
           break;

      default:
           usage (argv[0]);
           exit (1);
    }
  }

  if (optind < argc)
  {                             /* Some more parameters, assume is the server */
    strncpy (server, argv[optind], sizeof(server)-1);
    optind++;

    if (optind < argc)
         strncpy (password, argv[optind], sizeof(password)-1);
    else strcpy (password, "nurk");
  }
  else
    strcpy (server, "nemesis");

  fprintf (stderr, "Connecting to server \\\\%s ...\n", server);

  SMB_Init();
  sock_init();

  RFCNB_Set_Timeout (10);       /* Set a 10 second timeout on each of these */

  torture_mult_neg (server);
  
  /* Now connect again, and try something really bizare ... large number 
   * of bogus protocols ...
   */
  torture_long_neg (server);
  return (0);
}
