/*
 * Test out the RFCNB routines ...
 * 
 * Version 1.0
 * 
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <rfcnb.h>
#include <smblib.h>
#include <tcp.h>

void usage (char *prog)
{
  fprintf (stderr,
    "Usage: %s  [-du] [-a called-addr] <called-name> [calling-name]\n"
    "\t If [calling-name] is not specified, then current hostname is used.\n"
    "\t [called-addr] allows you to specify an ip address or DNS name\n"
    "\t to connect to.\n", prog);
  exit (0);
}

int main (int argc, char **argv)
{
  struct RFCNB_Con *con;
  char   calling[80] = "";
  char  *called      = NULL;
  char  *address     = NULL;
  int    opt, case_names = 0;

  while ((opt = getopt(argc,argv,"a:ud?")) != EOF)
        switch (opt)
        {
          case 'a':
               address = optarg;
               break;

          case 'u':
               case_names++;
               break;

          case 'd':
               dbug_init();
               break;

          case '?':
          default:
               usage (argv[0]);
        }

  argv += optind;
  argc -= optind;

  if (argc-- < 1 || (called = *argv++) == NULL)
     usage (NULL);

  SMB_Init();
  sock_init();

  if (argc >= 1)
  {
    strncpy (calling, *argv++, sizeof(calling)-1);
    argc--;
  }
  else if (gethostname(calling, sizeof(calling)) < 0)
  {
    perror ("gethostname");
    return (-1);
  }

  if (!address)
     address = called;

  fprintf (stderr, "Calling %s from %s (calling-addr %s)\n",
           called, calling, address);

  if (case_names)
  {
    strupr (called);
    strupr (calling);
  }

  con = RFCNB_Call (called, calling, address, 0);
  if (!con)
  {
    char err_string[512];

    RFCNB_Get_Error (err_string, sizeof(err_string));
    printf ("Error in calling: %s ...\n", err_string);
    return (1);
  }
  return (0);
}
